package environments

import (
	"context"
	"testing"
	"time"

	"github.com/avast/retry-go/v4"
	"github.com/stretchr/testify/assert"
)

type dummyEnvironments struct {
	Environments
	environment *Environment
	sleep       time.Duration
}

func (t *dummyEnvironments) Get(ctx context.Context, _ string, _ string) (*Environment, error) {
	if t.sleep != 0 {
		time.Sleep(t.sleep)
		if err := ctx.Err(); err != nil {
			return nil, err
		}
	}
	return t.environment, nil
}

func TestIsAvailable(t *testing.T) {
	tests := []struct {
		name        string
		environment *Environment
		wantErr     bool
	}{
		{
			"Environment has nil StateInfo: available",
			&Environment{ID: "env-id", SpaceID: "space-id"},
			false,
		},
		{
			"Environment state is StateReady: available",
			&Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StateReady}},
			false,
		},
		{
			"Environment state is StatePreparing: not available",
			&Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StatePreparing}},
			true,
		},
		{
			"Environment state is StateError: not available",
			&Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StateError}},
			true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			environments := &dummyEnvironments{environment: tt.environment}
			if err := IsAvailable(context.Background(), environments, "space-id", "master"); (err != nil) != tt.wantErr {
				t.Errorf("IsAvailable() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func TestWaitForAvailable(t *testing.T) {
	t.Run("Success", func(t *testing.T) {
		env := &Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StatePreparing}}
		environments := &dummyEnvironments{environment: env}

		// Имитируем завершение подготовки окружения через 1 секунду
		go func() {
			time.Sleep(1 * time.Second)
			env.StateInfo = &StateInfo{State: StateReady}
		}()

		err := WaitForAvailable(context.Background(), environments, "space-id", "master")
		assert.NoError(t, err)
	})
	t.Run("Overwritten retry options", func(t *testing.T) {
		env := &Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StatePreparing}}
		environments := &dummyEnvironments{environment: env}

		err := WaitForAvailable(context.Background(), environments, "space-id", "master", retry.Attempts(1))
		assert.Error(t, err, "За одну попытку окружение не стало доступным, получаем ошибку")
	})
	t.Run("External context was canceled", func(t *testing.T) {
		env := &Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StatePreparing}}
		environments := &dummyEnvironments{environment: env}

		ctx, cancel := context.WithCancel(context.Background())

		// Отменяем контекст через 1 секунду
		go func() {
			time.Sleep(1 * time.Second)
			cancel()
		}()

		err := WaitForAvailable(ctx, environments, "space-id", "master")
		assert.ErrorIs(t, err, context.Canceled)
	})
	t.Run("External context was deadline exceeded", func(t *testing.T) {
		env := &Environment{ID: "env-id", SpaceID: "space-id", StateInfo: &StateInfo{State: StatePreparing}}
		environments := &dummyEnvironments{environment: env}

		ctx, cancel := context.WithTimeout(context.Background(), 1*time.Second)
		defer cancel()

		err := WaitForAvailable(ctx, environments, "space-id", "master")
		assert.ErrorIs(t, err, context.DeadlineExceeded)
	})
}
